package com.jjt.risk.controller;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.jjt.biz.domain.BizObj;
import com.jjt.biz.domain.BizObjMetrics;
import com.jjt.biz.domain.BizObjMetricsData;
import com.jjt.biz.service.IBizObjMetricsDataService;
import com.jjt.biz.service.IBizObjMetricsService;
import com.jjt.biz.service.IBizObjService;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.http.HttpUtils;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.risk.domain.RiskAnalysis;
import com.jjt.risk.domain.RiskModel;
import com.jjt.risk.domain.RiskObj;
import com.jjt.risk.service.IRiskAnalysisService;
import com.jjt.risk.service.IRiskModelService;
import com.jjt.system.service.ISysConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 风险分析结果Controller
 *
 * @author jjt
 * @date 2024-09-12
 */
@Api(tags = "风险分析结果")
@RestController
@RequestMapping("/risk/analysis")
public class RiskAnalysisController extends BaseController {
    @Resource
    private IRiskAnalysisService riskAnalysisService;
    @Resource
    private IRiskModelService riskModelService;
    @Resource
    private IBizObjService objService;
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private IBizObjMetricsService objMetricsService;
    @Resource
    private IBizObjMetricsDataService dataService;
    @Resource
    private ISysConfigService configService;

    /**
     * 查询风险分析结果列表
     */
    @ApiOperation("查询风险分析结果列表")
    @PreAuthorize("@ss.hasPermi('risk:analysis:list')")
    @GetMapping("/list")
    public TableDataInfo list(RiskAnalysis riskAnalysis) {
        startPage();
        List<RiskAnalysis> list = riskAnalysisService.selectRiskAnalysisList(riskAnalysis);
        return getDataTable(list);
    }

    /**
     * 导出风险分析结果列表
     */
    @ApiOperation("导出风险分析结果列表")
    @PreAuthorize("@ss.hasPermi('risk:analysis:export')")
    @Log(title = "风险分析结果", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RiskAnalysis riskAnalysis) {
        List<RiskAnalysis> list = riskAnalysisService.selectRiskAnalysisList(riskAnalysis);
        ExcelUtil<RiskAnalysis> util = new ExcelUtil<RiskAnalysis>(RiskAnalysis.class);
        util.exportExcel(response, list, "风险分析结果数据");
    }

    /**
     * 获取风险分析结果详细信息
     */
    @ApiOperation("获取风险分析结果详细信息")
    @PreAuthorize("@ss.hasPermi('risk:analysis:query')")
    @GetMapping(value = "/{resultId}")
    public AjaxResult getInfo(@PathVariable("resultId") Long resultId) {
        riskAnalysisService.selectRiskAnalysisByResultId(resultId);
        return success();
    }

    @ApiOperation("立即分析")
    @GetMapping(value = "/atOnce/{riskId}")
    public AjaxResult atOnce(@PathVariable("riskId") Long riskId) throws UnsupportedEncodingException {
        String uri = configService.selectConfigByKey("risk.api.uri");
        RiskModel rm = riskModelService.selectRiskModelByRiskId(riskId);
        List<Long> objIds = new ArrayList<>();
        //找到对象
        if ("1".equals(rm.getObjType())) {
            //1.所有对象
            List<BizObj> list = objService.selectBizObjListByMetricsId(rm.getMetricsId());
            if (list != null) {
                objIds = list.stream().map(BizObj::getObjId).collect(Collectors.toList());
            }
        } else {
            //2.选择的对象
            List<RiskObj> objList = rm.getRiskObjList();
            if (objList != null) {
                objIds = objList.stream().map(RiskObj::getObjId).collect(Collectors.toList());
            }
        }

        BizObjMetrics queryOm = new BizObjMetrics();
        queryOm.setMetricsId(rm.getMetricsId());
        List<BizObjMetrics> allMetrics = objMetricsService.selectBizObjMetricsList(queryOm);
        Map<Long, List<BizObjMetrics>> objMap = allMetrics.stream().collect(Collectors.groupingBy(BizObjMetrics::getObjId));
        Date time = new Date();
//        rm.getRiskObjList()
        Random r = new Random();
        if ("3".equals(rm.getRiskType())) {
            List<Long> finalObjIds = objIds;
            List<BizObjMetrics> list = allMetrics.stream().filter(om -> finalObjIds.contains(om.getObjId())).collect(Collectors.toList());
            list = list.stream().sorted(Comparator.comparing(BizObjMetrics::getDValue).reversed()).collect(Collectors.toList());
            JSONObject jsonObject = new JSONObject();
            JSONArray dataArr = new JSONArray();
            JSONArray inputArr = new JSONArray();
            list.forEach(bom -> {
                dataArr.add(bom.getDValue().floatValue());
                JSONObject temp = new JSONObject();
                temp.put("id", bom.getObjMetricsId());
                temp.put("name", bom.getObjName() + "-" + bom.getMetricsName());
                temp.put("num", bom.getDValue().floatValue());
                inputArr.add(temp);
            });
            jsonObject.put("data", dataArr);
            JSONObject result = JSONObject.parseObject(HttpUtils.dataPost(uri + "/api/sas/diff", jsonObject));
            RiskAnalysis ra = new RiskAnalysis();
            ra.setRiskId(riskId);
            ra.setAnalyseTime(time);
            // 假设jsonArray是已有的JSONArray对象
            ra.setInput(inputArr.toString());
            ra.setImgPath("data:image/png;base64," + result.getJSONObject("data").getString("diff"));
            riskAnalysisService.insertRiskAnalysis(ra);
            return type3info(ra.getResultId());
        } else {
            LocalDateTime endTime = LocalDateTime.now();
            LocalDateTime beginTime = endTime.minusDays(7);
            Map<String, Object> currParams = new HashMap<>(16);
            currParams.put("beginTime", beginTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            currParams.put("endTime", endTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));

            endTime = beginTime;
            beginTime = endTime.minusDays(7);
            Map<String, Object> lastParams = new HashMap<>(16);
            lastParams.put("beginTime", beginTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            lastParams.put("endTime", endTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));

            for (Long objId : objIds) {
                List<BizObjMetrics> list = objMap.get(objId);

                list.forEach(om -> {
                    //查询2周的数据，调用python接口,并且存储？
                    BizObjMetricsData currQ = new BizObjMetricsData();
                    currQ.setObjMetricsId(om.getObjMetricsId());
                    currQ.setParams(currParams);
                    BizObjMetricsData lastQ = new BizObjMetricsData();
                    lastQ.setObjMetricsId(om.getObjMetricsId());
                    lastQ.setParams(lastParams);
                    List<BizObjMetricsData> currList = dataService.selectBizObjMetricsDataList(currQ);
                    List<BizObjMetricsData> lastList = dataService.selectBizObjMetricsDataList(lastQ);


                    JSONArray currArr = new JSONArray();
                    JSONArray lastArr = new JSONArray();
                    JSONArray sortArr = new JSONArray();
                    //用于补数据，因为需要本周的长度与上周相等
                    int dataSize = currList.size();
                    if (lastList.size() > dataSize) {
                        dataSize = lastList.size();
                    }

                    for (int i = 0; i < dataSize; i++) {
                        int currIndex = i;
                        int lastIndex = i;
                        if (currIndex >= currList.size()) {
                            currIndex = currIndex - currList.size();
                        }
                        if (lastIndex >= lastList.size()) {
                            lastIndex = lastIndex - lastList.size();
                        }
                        BizObjMetricsData curr = currList.get(currIndex);
                        currArr.add(curr.getdValue().floatValue());

                        BizObjMetricsData last = lastList.get(lastIndex);
                        lastArr.add(last.getdValue().floatValue());
                        sortArr.add(i + 1);
                    }

                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("level", rm.getConfidenceLevel().floatValue());
                    jsonObject.put("curr", currArr);
                    jsonObject.put("last", lastArr);
                    jsonObject.put("sort", sortArr);
                    JSONObject result = JSONObject.parseObject(HttpUtils.dataPost(uri + "/api/sas/trend", jsonObject));
                    JSONObject data = result.getJSONObject("data");
                    RiskAnalysis ra = new RiskAnalysis();
                    ra.setRiskId(riskId);
                    ra.setObjId(om.getObjMetricsId());
                    ra.setInput(jsonObject.toString());
                    ra.setResult(data.getString("trend"));
                    ra.setAvgValue(data.getString("ttest"));
                    ra.setNormalValid(data.getString("shapiro"));
                    ra.setVarianceValid(data.getString("levene"));
                    ra.setConfidenceLevel(rm.getConfidenceLevel().toString());
                    ra.setAnalyseTime(time);
                    riskAnalysisService.insertRiskAnalysis(ra);
                });
//                rm.getMetricsId()

            }
            return success();
        }

    }

    @ApiOperation("获取异类组件分析结果")
    @GetMapping(value = "/type3/result/{resultId}")
    public AjaxResult type3info(@PathVariable("resultId") Long resultId) {
        Map<String, Object> result = new HashMap<>(16);
        RiskAnalysis analysis = riskAnalysisService.selectRiskAnalysisByResultId(resultId);

        result.put("imgPath", analysis.getImgPath());
        result.put("top", JSONArray.parseArray(analysis.getInput()));
        return success(result);
    }

    @ApiOperation("根据对象获取曲线")
    @GetMapping(value = "/type3/trend/{id}")
    public AjaxResult type3trend(@PathVariable("id") Long id) {
        Map<String, Object> result = new HashMap<>(16);
        List<Map<String, Object>> list = new ArrayList();
        LocalDateTime ed = LocalDateTime.now();
        LocalDateTime st = ed.minusDays(7);
        List<Long> times = new ArrayList<>();
        List<Float> values = new ArrayList<>();

        Random r = new Random();
        DecimalFormat df = new DecimalFormat("#0.00");
        do {
            long time = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
            times.add(time);
            float f = r.nextFloat() * 100;
//                System.err.println(df.format(f));
            values.add(Float.parseFloat(df.format(f)));
            st = st.plusMinutes(15);
        } while (!st.isAfter(ed));
        result.put("times", times);
        result.put("values", values);
        return success(result);
    }

    /**
     * 新增风险分析结果
     */
    @ApiOperation("新增风险分析结果")
    @PreAuthorize("@ss.hasPermi('risk:analysis:add')")
    @Log(title = "风险分析结果", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RiskAnalysis riskAnalysis) {
        return toAjax(riskAnalysisService.insertRiskAnalysis(riskAnalysis));
    }

    /**
     * 修改风险分析结果
     */
    @ApiOperation("修改风险分析结果")
    @PreAuthorize("@ss.hasPermi('risk:analysis:edit')")
    @Log(title = "风险分析结果", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RiskAnalysis riskAnalysis) {
        return toAjax(riskAnalysisService.updateRiskAnalysis(riskAnalysis));
    }

    /**
     * 删除风险分析结果
     */
    @ApiOperation("删除风险分析结果")
    @PreAuthorize("@ss.hasPermi('risk:analysis:remove')")
    @Log(title = "风险分析结果", businessType = BusinessType.DELETE)
    @DeleteMapping("/{resultIds}")
    public AjaxResult remove(@PathVariable Long[] resultIds) {
        return toAjax(riskAnalysisService.deleteRiskAnalysisByResultIds(resultIds));
    }
}
